#include <sys/io.h>
#include <sys/time.h>
#include <assert.h>
#include <stdio.h>

#include <libada8.h>

/* ********************************************************************* */
/* ** Pin assignment of LPT-ports:                                    ** */
/* ** Addr\Bit:     7   6   5   4    3   2   1   0                    ** */
/* ** LPTBase       9   8   7   6    5   4   3   2                    ** */
/* ** LPTBase+1   11  10  12  13   15   x   x   x                    ** */
/* ** LPTBase+2     x   x   x   x  17  16 14  1                    ** */
/* ********************************************************************* */


#define LPT_BASE 0x378

#ifndef byte
typedef unsigned char byte;
#endif


static byte InputMask[] = 
{
  0x80,  // Select Input  1, disable IC5
  0x81,  // Select Input  2, disable IC5
  0x82,  // Select Input  3,   ...
  0x83,  // Select Input  4,   ...
  0x84,  // Select Input  5, 
  0x85,  // Select Input  6
  0x86,  // Select Input  7
  0x87,  // Select Input  8
  0x08,  // Select Input  9, disable IC4
  0x18,  // Select Input 10, disable IC4
  0x28,  // Select Input 11,   ...
  0x38,  // Select Input 12
};


ANALYZER::ANALYZER (int LPTBase) :
  LPTBase (LPTBase)
{
  // Get access to parallel part (LPTBase..LPTBase+2)
  assert (ioperm(LPTBase, 3, 1) == 0);
}


void ANALYZER::Write (int Value)
{
  byte Lo = Value & 0xFF;
  byte Hi = (Value >> 8) & 0x0F;

  /* Lock IC1 & IC2:
      => IC6#8 (C) = 0
        => 1) IC6#9 = 1  
              => Pin1 = 1  
	        => /Pin1 = 0
           2) IC6#10 = 1
              => IC6#3(A) = 1
                => IC6#1&2 = 0
	          => Pin14 = 0
		    => /Pin14 = 1
  */
  /* Store low-byte in IC3:
     => IC6#11(D) = 1
       => (IC6#12 = 0) OR (IC6#13 = 0)
         => Pin16 = 0 
                           => IC6#3(A) = 0
		             => IC6#1&2 = 1
		               => Pin14 = 1
			         => /Pin14 = 0
  */
  /* Generate accept impulse for IC1 & IC2:
     => IC6#8(C) = 1
       => (IC6#9 = 0) OR (IC6#10 = 0)
         => Pin1 = 0
           => /Pin1 = 1
                          => IC6#3(A) = 0
			    => IC6#1&2 = 1
			      => Pin14 = 1
			        => /Pin14 = 0
  */

  // Lock IC1 and IC2 (IC6#8(C), enable IC3 (IC6#11(D))
  outb (0x02, LPT_BASE+2);  

  // Set low byte of value (store in IC3)
  outb (Lo, LPT_BASE);

  // lock IC6(C) and IC6(D)
  outb (0x06, LPT_BASE+2);

  // take care locking is active
  Delay (5);                 

  // Set high byte (4 bits) of value
  outb (Hi, LPT_BASE);

  // Activate IC1 & IC2 for output
  outb (0x0F, LPT_BASE+2);

  // lock IC1, IC2 and IC3 again
  outb (0x06, LPT_BASE+2);
}


int ANALYZER::Read (int Mask /* = 0x0FFF */)
{
  byte Value;
  int  count, Result = 0;

  /* Read Data from Pin 10:
     => 1) IC1 & IC2 are locked
	   => /Pin1 = 0 AND /Pin14=1 (see above)
        2) IC3 write through
           => IC6#11(D) = 1 
              => Pin16 = 0 OR /Pin14 = 0  (see above)
        3) D1..D8 select input signal
	   lowbyte:
           => (D4 = 0) AND (D8 = 1) AND (D1..D3) select
	   highbyte:
	   => (D8 = 0) AND (D4 = 1) AND (D5..D6) select
  */

  // Enable IC3 and lock IC1 & IC2
  outb (0x02, LPT_BASE+2);   
  for (count = 11; count >= 0; count--)
  {
    // shift left for adding next result-bit
    Result <<= 1;

    if (Mask & (1 << count))
    {
      // Set input selection for IC3
      outb (InputMask[count], LPT_BASE);

      // Wait for multiplexer (30ns at 5V) and comparator (at least 6 sec).
      // Well, oscilloscope says: The circuit need about 8-10msec from 
      // selection till data is surely available. So this value might not be
      // enough.
      Delay (400); 

      // Read value at Pin 10
      Value = inb (LPT_BASE+1);

      if ((Value & 0x40) == 0)
	Result += 1;
    }
  }

  // Lock IC1..IC3, again
  outb (0x06, LPT_BASE+2);

  return Result;
}


// simple delay loop because usleep does not work correctly
void ANALYZER::Delay (long uSec)
{
  struct timeval Start, Time;

  gettimeofday (&Start, NULL);
  do
    gettimeofday (&Time, NULL);
  while ((Time.tv_sec - Start.tv_sec) * 1000000 + 
	 (Time.tv_usec - Start.tv_usec) < uSec);
}

